(******************************************************************************
 *                                                                            *
 *      ,  2002                                 *
 *                                                                            *
 *  :       main.pas                                                      *
 *  :   DirectInput     *
 *                                                                            *
 ******************************************************************************)
unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, StdCtrls,
  ExtCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    gb1: TGroupBox;
    gb2: TGroupBox;
    gb3: TGroupBox;
    lbInfo: TLabel;
    lb1: TLabel;
    lb2: TLabel;
    lb3: TLabel;
    lb4: TLabel;
    lb5: TLabel;
    lb6: TLabel;
    lb7: TLabel;
    lb8: TLabel;
    lbXAxis: TLabel;
    lbYAxis: TLabel;
    lbZAxis: TLabel;
    lbXRot: TLabel;
    lbYRot: TLabel;
    lbZRot: TLabel;
    lbSlider1: TLabel;
    lbSlider2: TLabel;
    lbIndex: TLabel;
    lbButtons: TLabel;
    btnClose: TButton;
    imView: TImage;
    Bevel1: TBevel;
    procedure btnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure Idle( Sender: TObject; var Done: Boolean );
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  DirectInput8;




//------------------------------------------------------------------------------
//    
//------------------------------------------------------------------------------
var
  lpDI8:          IDirectInput8       = nil;
  lpDIGameDevice: IDirectInputDevice8 = nil;




//------------------------------------------------------------------------------
// :      EnumJoysticksCallback()
// :      
//------------------------------------------------------------------------------
function EnumJoysticksCallback( var lpddi: TDIDeviceInstanceA; pvRef: Pointer ):
   Integer; stdcall;
var
  didc: TDIDevCaps;
begin
  Result := DIENUM_CONTINUE;

  //    GUID       
  if FAILED( lpDI8.CreateDevice( lpddi.guidInstance, lpDIGameDevice, nil ) ) then
     Exit;
  lpDIGameDevice._AddRef;

  //    (   )
  case GET_DIDEVICE_TYPE( lpddi.dwDevType ) of
    DI8DEVTYPE_JOYSTICK: Form1.lbInfo.Caption := ' ';
    DI8DEVTYPE_GAMEPAD:  Form1.lbInfo.Caption := ' ';
    DI8DEVTYPE_DRIVING:  Form1.lbInfo.Caption := ' ';
    DI8DEVTYPE_FLIGHT:   Form1.lbInfo.Caption := ' ';
  end;

  //   TDIDEVCAPS,     
  //     
  ZeroMemory( @didc, SizeOf( TDIDEVCAPS ) );
  didc.dwSize := SizeOf( TDIDEVCAPS );

  //    
  if FAILED( lpDIGameDevice.GetCapabilities( didc ) ) then
     Exit;

  //   
  Form1.lbInfo.Caption := Form1.lbInfo.Caption + lpddi.tszProductName + ' (' +
                          IntToStr( didc.dwAxes ) + ' , '+
                          IntToStr( didc.dwButtons ) + ' ).';
  Result := DIENUM_STOP;
end;




//------------------------------------------------------------------------------
// :      EnumAxecCallback()
// :         
//------------------------------------------------------------------------------
function EnumAxecCallback( var lpddoi: TDIDeviceObjectInstanceA;
                           pvRef: Pointer ): Integer; stdcall;
//  ,      if guid1 = guid2..., 
//    ,    GUID
function Coincide( g1, g2: TGUID ): Boolean;
begin
  Result := FALSE;

  if ( g1.D1    = g2.D1 )   and
     ( g1.D2    = g2.D2 )   and
     ( g1.D3    = g2.D3 )   and
     ( g1.D4[0] = g2.D4[0]) and
     ( g1.D4[1] = g2.D4[1]) and
     ( g1.D4[2] = g2.D4[2]) and
     ( g1.D4[3] = g2.D4[3]) and
     ( g1.D4[4] = g2.D4[4]) and
     ( g1.D4[5] = g2.D4[5]) and
     ( g1.D4[6] = g2.D4[6]) and
     ( g1.D4[7] = g2.D4[7]) then
  Result := TRUE;
end; // Coincide()

var
  diprgn: TDIPROPRANGE;
begin
  Result := DIENUM_STOP;

  //   TDIPROPRANGE
  //    
  ZeroMemory( @diprgn, SizeOf( TDIPROPRANGE ) );
  diprgn.diph.dwSize       := SizeOf( TDIPROPRANGE );
  diprgn.diph.dwHeaderSize := SizeOf( TDIPROPHEADER );

  diprgn.diph.dwHow := DIPH_BYID;     //    
  diprgn.diph.dwObj := lpddoi.dwType; //   
  diprgn.lMin       := -1000;         //    
  diprgn.lMax       := +1000;         //    

  //      
  if FAILED( lpDIGameDevice.SetProperty( DIPROP_RANGE, diprgn.diph ) ) then
     Exit;

  //      ,    
  with Form1 do
  begin
    if Coincide( lpddoi.guidType, GUID_XAxis ) then
    begin
      lb1.Enabled := TRUE;
      lbXAxis.Enabled := TRUE;
    end;

    if Coincide( lpddoi.guidType, GUID_YAxis ) then
    begin
      lb2.Enabled := TRUE;
      lbYAxis.Enabled := TRUE;
    end;

    if Coincide( lpddoi.guidType, GUID_ZAxis ) then
    begin
      lb3.Enabled := TRUE;
      lbZAxis.Enabled := TRUE;
    end;

    if Coincide( lpddoi.guidType, GUID_RxAxis ) then
    begin
      lb4.Enabled := TRUE;
      lbXRot.Enabled := TRUE;
    end;

    if Coincide( lpddoi.guidType, GUID_RyAxis ) then
    begin
      lb5.Enabled := TRUE;
      lbYRot.Enabled := TRUE;
    end;

    if Coincide( lpddoi.guidType, GUID_RzAxis ) then
    begin
      lb6.Enabled := TRUE;
      lbZRot.Enabled := TRUE;
    end;

    if Coincide( lpddoi.guidType, GUID_Slider ) then
    begin
      lb7.Enabled := TRUE;
      lbSlider1.Enabled := TRUE;
      lb8.Enabled := TRUE;
      lbSlider2.Enabled := TRUE;
    end;
  end;

  Result := DIENUM_CONTINUE;
end;




//------------------------------------------------------------------------------
// :      InitDirectInput()
// :    DirectInput  
//------------------------------------------------------------------------------
function InitDirectInput( hWnd: HWND ): Boolean;
begin
  Result := FALSE;

  //   DirectInput
  if FAILED( DirectInput8Create( GetModuleHandle( 0 ), DIRECTINPUT_VERSION,
                                 IID_IDirectInput8, lpDI8, nil ) ) then
     Exit;
  lpDI8._AddRef();

  //       
  if FAILED( lpDI8.EnumDevices( DI8DEVCLASS_GAMECTRL, EnumJoysticksCallback, nil,
                                DIEDFL_ATTACHEDONLY ) ) then
     Exit;

  // ,     
  if lpDIGameDevice = nil then
     Exit;

  //     " " -  
  //   ,       -
  //       .    DInput,   
  //   TDIJOYSTATE   GetDeviceState(). 
  //    TDIJOYSTATE2,  . .    
  //    ,    TDIJOYSTATE.
  if FAILED( lpDIGameDevice.SetDataFormat( @c_dfDIJoystick ) ) then
     Exit;

  //   ,  DInput ,   
  //         DInput.
  if FAILED( lpDIGameDevice.SetCooperativeLevel( hWnd, DISCL_BACKGROUND or
                                                       DISCL_EXCLUSIVE ) ) then
     Exit;

  //          .
  // :        ,   , 
  //     (,   . .)
  if FAILED( lpDIGameDevice.EnumObjects( EnumAxecCallback, nil, DIDFT_AXIS ) ) then
     Exit;

  //   
  lpDIGameDevice.Acquire();

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      ReleaseDirectInput()
// :    DirectInput
//------------------------------------------------------------------------------
procedure ReleaseDirectInput();
begin
  //       
  if lpDIGameDevice <> nil then
  begin
    lpDIGameDevice.Unacquire(); //  
    lpDIGameDevice._Release();
    lpDIGameDevice := nil;
  end;

  //    DirectInput ( )
  if lpDI8 <> nil then
  begin
    lpDI8._Release();
    lpDI8 := nil;
  end;
end;




//------------------------------------------------------------------------------
// :      UpdateGameDeviceState()
// :        
//------------------------------------------------------------------------------
function UpdateGameDeviceState(): Boolean;
var
  js: TDIJOYSTATE;
  i,
  nXPos,
  nYPos:  Integer;
begin
  Result := FALSE;

  //   
  if FAILED( lpDIGameDevice.Poll() ) then
     lpDIGameDevice.Acquire();

  //    ,     js
  if FAILED( lpDIGameDevice.GetDeviceState( SizeOf( TDIJOYSTATE ), @js ) ) then
     Exit;

  //    
  with Form1 do
  begin
    lbXAxis.Caption := IntToStr( js.lX );
    lbYAxis.Caption := IntToStr( js.lY );
    lbZAxis.Caption := IntToStr( js.lZ );
    lbXRot.Caption  := IntToStr( js.lRx );
    lbYRot.Caption  := IntToStr( js.lRy );
    lbZRot.Caption  := IntToStr( js.lRz );

    lbSlider1.Caption := IntToStr( js.rglSlider[ 0 ] );
    lbSlider2.Caption := IntToStr( js.rglSlider[ 1 ] );

    lbButtons.Caption := '';

    //      
    for i := 0 to 127 do // DirectInput8   128 
    begin
      if js.rgbButtons[ i ] = $80 then //   
      if i <= 9 then lbButtons.Caption := lbButtons.Caption + Format( '0%d ', [ i + 1 ] )
                else lbButtons.Caption := lbButtons.Caption + Format( '%d ', [ i + 1 ] );
    end;

    //     
    nXPos := Round( js.lX / 22 ) + 50;
    nYPos := Round( js.lY / 22 ) + 50;

    //  
    with imView.Canvas do
    begin
      //   
      Brush.Color := clWhite;
      FillRect( Canvas.ClipRect );

      //  
      MoveTo( nXPos - 4, nYPos );
      LineTo( nXPos + 5, nYPos );

      MoveTo( nXPos, nYPos + 4 );
      LineTo( nXPos, nYPos - 5 );
    end;
  end;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      TForm1.Idle()
// :     
//------------------------------------------------------------------------------
procedure TForm1.Idle( Sender: TObject; var Done: Boolean );
begin
  //      
  if not UpdateGameDeviceState() then
  begin
    MessageBox( Form1.Handle, '  !',
                '!', MB_ICONHAND);
    Form1.Close();
  end;

  Done := FALSE;
end;




//------------------------------------------------------------------------------
// :      TForm1.FormCreate()
// :   DirectInput   
//------------------------------------------------------------------------------
procedure TForm1.FormCreate(Sender: TObject);
begin
  if not InitDirectInput( Form1.Handle ) then
  begin
    MessageBox( Form1.Handle, '   DirectInput!' + #13 +
                ',         .',
                '!', MB_ICONHAND);
    ReleaseDirectInput();
    Halt;
  end;

  //   Idle-.  TTimer   
  //    DirectInput.
  Application.OnIdle := Idle;
end;




//------------------------------------------------------------------------------
// :      TForm1.btnCloseClick()
// :  
//------------------------------------------------------------------------------
procedure TForm1.btnCloseClick(Sender: TObject);
begin
  Form1.Close();
end;




//------------------------------------------------------------------------------
// :      TForm1.FormDestroy()
// :      
//------------------------------------------------------------------------------
procedure TForm1.FormDestroy(Sender: TObject);
begin
   ReleaseDirectInput();
end;

end.
