(******************************************************************************
 *                                                                            *
 *      ,  2002                                 *
 *                                                                            *
 *  :       main.pas                                                      *
 *  :   DirectInput           *
 *                                                                            *
 ******************************************************************************)
unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, ComCtrls,
  StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    gb1: TGroupBox;
    gb2: TGroupBox;
    gb3: TGroupBox;
    lbRemark: TLabel;
    imView: TImage;
    rbWM: TRadioButton;
    rgDI8: TRadioButton;
    lbKeys: TLabel;
    lbIndex: TLabel;
    btnClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure Hook( var Msg: TMsg; var Handled: Boolean );
    procedure Idle( Sender: TObject; var Done: Boolean );
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  DirectInput8;




//------------------------------------------------------------------------------
//    
//------------------------------------------------------------------------------
var
  lpDI8:        IDirectInput8       = nil;
  lpDIKeyboard: IDirectInputDevice8 = nil;

  nXPos,
  nYPos:         Integer;




//------------------------------------------------------------------------------
// :      InitDirectInput()
// :    DirectInput  
//------------------------------------------------------------------------------
function InitDirectInput( hWnd: HWND ): Boolean;
begin
  Result := FALSE;

  //    DirectInput
  if FAILED( DirectInput8Create( GetModuleHandle( 0 ), DIRECTINPUT_VERSION,
                                 IID_IDirectInput8, lpDI8, nil ) ) then
     Exit;
  lpDI8._AddRef();

  //      
  if FAILED( lpDI8.CreateDevice( GUID_SysKeyboard, lpDIKeyboard, nil ) ) then
     Exit;
  lpDIKeyboard._AddRef();

  //     " ".  -
  //      ,   
  // c_dfDIKeyboard  ,        
  if FAILED( lpDIKeyboard.SetDataFormat( @c_dfDIKeyboard ) ) then
     Exit;

  //   .      DirectX SDK
  if FAILED( lpDIKeyboard.SetCooperativeLevel( hWnd, DISCL_BACKGROUND or
                                                     DISCL_NONEXCLUSIVE ) ) then
     Exit;

  //  
  lpDIKeyboard.Acquire();

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      ReleaseDirectInput()
// :    DirectInput
//------------------------------------------------------------------------------
procedure ReleaseDirectInput();
begin
  //      
  if lpDIKeyboard <> nil then //   if Assigned( DIKeyboard )
  begin
    lpDIKeyboard.Unacquire(); //  
    lpDIKeyboard._Release();
    lpDIKeyboard := nil;
  end;

  //     DirectInput
  if lpDI8 <> nil then
  begin
    lpDI8._Release();
    lpDI8 := nil;
  end;
end;




//------------------------------------------------------------------------------
// :      UpdateKeyboardState()
// :     DirectInput
//------------------------------------------------------------------------------
function UpdateKeyboardState(): Boolean;
var
  bKeyBuffer: array [0..255] of Byte;
  i:          Integer;

  hr:         HRESULT;
begin
  Result := FALSE;

  //    ,    -
  if lpDIKeyboard.GetDeviceState( SizeOf( bKeyBuffer ), @bKeyBuffer ) = DIERR_INPUTLOST then
  begin
    //  
    lpDIKeyboard.Acquire();
    //   
    if FAILED( lpDIKeyboard.GetDeviceState( SizeOf( bKeyBuffer ), @bKeyBuffer ) ) then
       Exit;
  end;

  //   
  if bKeyBuffer[ DIK_NUMPAD4 ] = $080 then Dec( nXPos );
  if bKeyBuffer[ DIK_NUMPAD6 ] = $080 then Inc( nXPos );
  if bKeyBuffer[ DIK_NUMPAD8 ] = $080 then Dec( nYPos );
  if bKeyBuffer[ DIK_NUMPAD2 ] = $080 then Inc( nYPos );

  //     
  with Form1.lbKeys do
  begin
    Caption := '';

    for i := 0 to 255 do
    if bKeyBuffer[ i ] = $080 then
    if i <= 9 then Caption := Caption + Format( '0%d ', [ i ] )
              else Caption := Caption + Format( '%d ', [ i ] );
  end;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      TForm1.Hook()
// :       
//------------------------------------------------------------------------------
procedure TForm1.Hook( var Msg: TMsg; var Handled: Boolean );
var
  i: Integer;
begin
  if Msg.message <> WM_KEYDOWN then
     Exit;

  //   
  case Msg.wParam of
     VK_NUMPAD4: Dec( nXPos );
     VK_NUMPAD6: Inc( nXPos );
     VK_NUMPAD8: Dec( nYPos );
     VK_NUMPAD2: Inc( nYPos );
  end;

  //    
  with Form1.lbKeys do
  begin
    Caption := '';

    //   for i := 0 to 255 do ...   
    // WM_KEYDOWN        -  
    //  .    ,   Windows 
    //  GetKeyboardState(),      
    if Msg.wParam <= 9 then Caption := Caption + Format( '0%d ', [ Msg.wParam ] )
                       else Caption := Caption + Format( '%d ', [ Msg.wParam ] );
  end;

  //    
  Handled := TRUE;
end;




//------------------------------------------------------------------------------
// :      TForm1.Idle()
// :     
//------------------------------------------------------------------------------
procedure TForm1.Idle( Sender: TObject; var Done: Boolean );
var
  i: Integer;
begin
  if rbWM.Checked then Application.OnMessage := Hook
  else
  begin
    Application.OnMessage := nil;

    //      
    if not UpdateKeyboardState() then
    begin
       MessageBox( Form1.Handle, '  !',
                  '!', MB_ICONHAND );
       Form1.Close();
    end;
  end;

  //      
  if nXPos < 0        then nXPos := 0;
  if nXPos + 10 > 140 then nXPos := 130;
  if nYPos < 0        then nYPos := 0;
  if nYPos + 10 > 140 then nYPos := 130;

  //  
  with imView.Canvas do
  begin
    FillRect( Canvas.ClipRect );

    Brush.Color := clRed;
    Rectangle( nXPos, nYPos, nXPos + 10, nYPos + 10 );
    Brush.Color := clWhite;
  end;

  Done := FALSE;
end;




//------------------------------------------------------------------------------
// :      TForm1.FormCreate()
// :   DirectInput   
//------------------------------------------------------------------------------
procedure TForm1.FormCreate(Sender: TObject);
begin
  if not InitDirectInput( Form1.Handle ) then
  begin
    MessageBox( Form1.Handle, '   DirectInput!',
                '!', MB_ICONHAND );
    ReleaseDirectInput();
    Halt;
  end;

  //  UI   
  lbKeys.Caption := '';

  //   Idle-.  TTimer   
  //    DirectInput
  Application.OnIdle := Idle;
end;




//------------------------------------------------------------------------------
// :      TForm1.btnCloseClick()
// :  
//------------------------------------------------------------------------------
procedure TForm1.btnCloseClick(Sender: TObject);
begin
  Form1.Close();
end;




//------------------------------------------------------------------------------
// :      TForm1.FormDestroy()
// :      
//------------------------------------------------------------------------------
procedure TForm1.FormDestroy(Sender: TObject);
begin
  ReleaseDirectInput();
end;

end.
